﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;

namespace bytutorial.com.minifier
{
    class Program
    {
        private static List<string> sourceDirectories = new List<string>();
        private static string ignoreFileNames = "";
        private static string minifyFiles = "";
        private static string filePatterns = "";

        static void Main(string[] args)
        {
            //get the configuration files
            GetSettingValues();
        }


        //Method to getting the settings value from setting.config file
        static void GetSettingValues()
        {
            try
            {
                XmlDocument doc = new XmlDocument();
                string xmlText = System.IO.File.ReadAllText(AppDomain.CurrentDomain.BaseDirectory + "/setting.config");
                doc.LoadXml(xmlText.Substring(xmlText.IndexOf(Environment.NewLine)));
                XmlNode nodeParent = doc.SelectSingleNode("/configuration");
                if (nodeParent != null)
                {
                    XmlNodeList childNodes = nodeParent.ChildNodes;
                    foreach (XmlNode node in childNodes)
                    {
                        switch (node.Name)
                        {
                            case "SourceDirectories":
                                XmlNodeList sourceDirectoryNodes = node.ChildNodes;
                                foreach (XmlNode dbNode in sourceDirectoryNodes)
                                {
                                    sourceDirectories.Add(dbNode.InnerText);
                                }
                                break;
                            case "Settings":
                                XmlNodeList settingNodes = node.ChildNodes;
                                foreach (XmlNode nodeSetting in settingNodes)
                                {
                                    switch (nodeSetting.Name)
                                    {
                                        case "FilePatterns":
                                            filePatterns = nodeSetting.InnerText;
                                            break;
                                        case "IgnoreFileNames":
                                            ignoreFileNames = nodeSetting.InnerText;
                                            break;
                                    }
                                }
                                break;
                        }
                    }
                }

                //show the configuration summary settings
                WriteLine(">>> Configuration settings have been read successfully.");
                WriteLine(">>> File Patterns: " + filePatterns);
                WriteLine(">>> Ignore Files: " + ignoreFileNames);

                //empty line
                WriteLine("");

                ignoreFileNames = "," + ignoreFileNames + ",";
                List<FileInfo> allFiles = new List<FileInfo>();

                foreach (string dir in sourceDirectories)
                {
                    //single extension only
                    if (filePatterns.IndexOf(",") == -1)
                    {
                        AddFiles(allFiles, dir, minifyFiles);
                    }
                    else
                    {
                        string[] patternArray = filePatterns.Split(',');
                        foreach (string filePattern in patternArray)
                        {
                            AddFiles(allFiles, dir, filePattern);
                        }
                    }

                }

                int minifyCounter = 0;
                foreach (FileInfo file in allFiles)
                {
                    if (ignoreFileNames.IndexOf("," + file.Name.ToLower() + ",") == -1)
                    {
                        //Read the content of the file
                        string text = System.IO.File.ReadAllText(file.FullName);

                        //re-write back the minify content to the same file name
                        using (StreamWriter writetext = new StreamWriter(file.FullName, false))
                        {
                            StringBuilder sb = new StringBuilder();
                            writetext.Write(Utils.minifyContent(text, file.Extension.ToLower() == ".html"));
                        }

                        //Show the minified file
                        WriteLine("Minify File: " + file.FullName);

                        //Add the minify counter
                        minifyCounter++;
                    }
                }
                ReadLine("Total minify files: " + minifyCounter.ToString() + (minifyCounter > 1 ? " files." : " file."));
            }
            catch (Exception ex)
            {
                WriteLine(">>> Error reading configuration: " + ex.Message.ToString());
                ReadLine();
            }
        }

        //Method to add files
        static void AddFiles(List<FileInfo> allFiles, string directoryPath, string extension)
        {
            FileInfo[] files = Utils.GetFiles(directoryPath, extension);
            if (files.Count() > 0)
            {
                foreach (FileInfo objFile in files)
                {
                    allFiles.Add(objFile);
                }
            }
        }

        //Method to write console line
        static void WriteLine(string message)
        {
            Console.WriteLine(message);
        }

        //Method to read console line
        static void ReadLine(string message = "")
        {
            WriteLine(message);
            Console.ReadLine();
        }
    }
}
