﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace bytutorial.com.minifier
{
    public static class Utils
    {
        static void FindFiles(List<FileInfo> allFiles, string directoryPath, string extension)
        {
            FileInfo[] files = Utils.GetFiles(directoryPath, extension);
            if (files.Count() > 0)
            {
                foreach (FileInfo objFile in files)
                {
                    allFiles.Add(objFile);
                }
            }
        }

        static void WriteLine(string message)
        {
            Console.WriteLine(message);
        }

        public static void ReadLine(string message = "")
        {
            WriteLine(message);
            Console.ReadLine();
        }

        //Method to get files
        public static FileInfo[] GetFiles(string fullPath, string extension)
        {
            DirectoryInfo objDirectoryInfo = new DirectoryInfo(fullPath);
            FileInfo[] files = null;
            if (extension.Length > 0)
            {
                files = objDirectoryInfo.GetFiles(extension, SearchOption.AllDirectories);
            }
            else
            {
                files = objDirectoryInfo.GetFiles("*.*", SearchOption.AllDirectories);
            }
            return files;
        }

        //Method to minify content
        public static string minifyContent(string content, bool isHTML)
        {
            //strip the comment
            content = isHTML ? RemoveHTMLComments(content) : RemoveCSSJSComments(content);

            var sb = new StringBuilder();
            foreach (char ch in content)
            {
                if (ch != '\t' && ch != '\n' && ch != '\r')
                {
                    sb.Append(ch);
                }
            }

            return content;
        }

        public static string RemoveCSSJSComments(string text)
        {
            //replace single comment
            text = Regex.Replace(text, @"[^:]\/\/.*", "");

            //replace multiple comments
            text = Regex.Replace(text, @"\/\*[\s\S]*?\*\/|([^:]|^)\/\/.*$", "");

            //replace double or more spaces
            text = Regex.Replace(text.ToString(), @"\s+", " ");

            return text;
        }

        static string RemoveHTMLComments(string text)
        {
            //replace html comment tag
            var re = @"<!--.*?-->";
            return Regex.Replace(text, re, "");
        }
    }
}
